@@ -16,6 +16,7 @@ import numpy as np
 import numpy.ma as ma
 import numpy.testing as npt
 import pandas as pd
+import pytest
 import scipy.sparse as sps
 import theano
 import theano.sparse as sparse
@@ -25,24 +26,23 @@ import pymc3 as pm
 
 
 class TestHelperFunc:
-    def test_pandas_to_array(self):
+    @pytest.mark.parametrize("input_dtype", ["int32", "int64", "float32", "float64"])
+    def test_pandas_to_array(self, input_dtype):
         """
         Ensure that pandas_to_array returns the dense array, masked array,
         graph variable, TensorVariable, or sparse matrix as appropriate.
         """
         # Create the various inputs to the function
-        sparse_input = sps.csr_matrix(np.eye(3))
-        dense_input = np.arange(9).reshape((3, 3))
+        sparse_input = sps.csr_matrix(np.eye(3)).astype(input_dtype)
+        dense_input = np.arange(9).reshape((3, 3)).astype(input_dtype)
 
         input_name = "input_variable"
         theano_graph_input = tt.as_tensor(dense_input, name=input_name)
-
         pandas_input = pd.DataFrame(dense_input)
 
         # All the even numbers are replaced with NaN
-        missing_pandas_input = pd.DataFrame(
-            np.array([[np.nan, 1, np.nan], [3, np.nan, 5], [np.nan, 7, np.nan]])
-        )
+        missing_numpy_input = np.array([[np.nan, 1, np.nan], [3, np.nan, 5], [np.nan, 7, np.nan]])
+        missing_pandas_input = pd.DataFrame(missing_numpy_input)
         masked_array_input = ma.array(dense_input, mask=(np.mod(dense_input, 2) == 0))
 
         # Create a generator object. Apparently the generator object needs to
@@ -71,7 +71,7 @@ class TestHelperFunc:
 
         # Check function behavior when using masked array inputs and pandas
         # objects with missing data
-        for input_value in [masked_array_input, missing_pandas_input]:
+        for input_value in [missing_numpy_input, masked_array_input, missing_pandas_input]:
             func_output = func(input_value)
             assert isinstance(func_output, ma.core.MaskedArray)
             assert func_output.shape == input_value.shape
@@ -79,8 +79,20 @@ class TestHelperFunc:
 
         # Check function behavior with Theano graph variable
         theano_output = func(theano_graph_input)
-        assert isinstance(theano_output, theano.gof.graph.Variable)
-        assert theano_output.owner.inputs[0].name == input_name
+        assert isinstance(theano_output, theano.graph.basic.Variable)
+        npt.assert_allclose(theano_output.eval(), theano_graph_input.eval())
+        intX = pm.theanof._conversion_map[theano.config.floatX]
+        if dense_input.dtype == intX or dense_input.dtype == theano.config.floatX:
+            assert theano_output.owner is None  # func should not have added new nodes
+            assert theano_output.name == input_name
+        else:
+            assert theano_output.owner is not None  # func should have casted
+            assert theano_output.owner.inputs[0].name == input_name
+
+        if "float" in input_dtype:
+            assert theano_output.dtype == theano.config.floatX
+        else:
+            assert theano_output.dtype == intX
 
         # Check function behavior with generator data
         generator_output = func(square_generator)
